// TextureFill.cpp : Defines the entry point for the console application.
//

#include "stdafx.h"

#include <cuda.h>
#include <cuda_runtime.h>

extern "C" void cudaZero(float *slices);
extern "C" void cudaSample4(float *slices, cudaArray *ptr1, cudaArray *ptr2, cudaArray *ptr3, cudaArray *ptr4);

int _tmain(int argc, _TCHAR* argv[])
{
	int mVoxelsX = 1024;
	int mVoxelsY = 1024;
	int mVoxelsZ = 128;

	int mPixelsX = 64;
	int mPixelsY = 8;

	int devices = 0;
	cudaGetDeviceCount(&devices);

	for(int device = 0; device < devices; device++)
	{
		cudaSetDevice(device);

		cudaSetDeviceFlags(cudaDeviceBlockingSync);

		cudaDeviceProp prop;
		cudaGetDeviceProperties(&prop, device);

		printf("%s\n", prop.name);

		cudaEvent_t event1, event2;

		cudaEventCreateWithFlags(&event1, cudaEventBlockingSync);
		cudaEventCreateWithFlags(&event2, cudaEventBlockingSync);

		float *ptr = 0;
		cudaMalloc((void **)&ptr, mVoxelsX * mVoxelsY * mVoxelsZ * sizeof(float));

		void *temp = 0;
		cudaMallocHost((void **)&temp, mPixelsX * mPixelsY * sizeof(float));
		memset(temp, 0, mPixelsX * mPixelsY * sizeof(float));

		cudaArray *ptr1 = 0;
		cudaMallocArray(&ptr1, &cudaCreateChannelDesc<float>(), mPixelsX, mPixelsY);
		cudaMemcpyToArray(ptr1, 0, 0, temp, mPixelsX * mPixelsY * sizeof(float), cudaMemcpyHostToDevice);

		cudaArray *ptr2 = 0;
		cudaMallocArray(&ptr2, &cudaCreateChannelDesc<float>(), mPixelsX, mPixelsY);
		cudaMemcpyToArray(ptr2, 0, 0, temp, mPixelsX * mPixelsY * sizeof(float), cudaMemcpyHostToDevice);

		cudaArray *ptr3 = 0;
		cudaMallocArray(&ptr3, &cudaCreateChannelDesc<float>(), mPixelsX, mPixelsY);
		cudaMemcpyToArray(ptr3, 0, 0, temp, mPixelsX * mPixelsY * sizeof(float), cudaMemcpyHostToDevice);

		cudaArray *ptr4 = 0;
		cudaMallocArray(&ptr4, &cudaCreateChannelDesc<float>(), mPixelsX, mPixelsY);
		cudaMemcpyToArray(ptr4, 0, 0, temp, mPixelsX * mPixelsY * sizeof(float), cudaMemcpyHostToDevice);

		cudaFreeHost(temp);

		float ms;

		cudaEventRecord(event1);

		cudaZero(ptr);

		cudaEventRecord(event2);

		cudaEventSynchronize(event2);

		cudaEventElapsedTime(&ms, event1, event2);

		printf("cudaZero                        %0.3f ms\t%0.3f GB/s\n", ms, 4.0f / (1000000.0f * ms) * mVoxelsX * mVoxelsY * mVoxelsZ);

		cudaEventRecord(event1);

		cudaSample4(ptr, ptr1, ptr2, ptr3, ptr4);

		cudaEventRecord(event2);

		cudaEventSynchronize(event2);

		cudaEventElapsedTime(&ms, event1, event2);

		printf("cudaSample4                     %0.3f ms\t%0.3f GB/s\t%0.3lf GT/s\n", ms, 8.0f / (1000000.0f * ms) * mVoxelsX * mVoxelsY * mVoxelsZ, 4.0f / (1000000.0f * ms) * mVoxelsX * mVoxelsY * mVoxelsZ);

		cudaFreeArray(ptr1);
		cudaFreeArray(ptr2);
		cudaFreeArray(ptr3);
		cudaFreeArray(ptr4);

		cudaFree(ptr);

		cudaEventDestroy(event1);
		cudaEventDestroy(event2);

		cudaThreadExit();
	}

	return 0;
}

